import bpy
import time
import uuid  # For generating unique IDs
from bpy.props import (
    BoolProperty,
    PointerProperty,
    CollectionProperty,
    StringProperty,
    IntProperty,
    FloatProperty,
)
from bpy.types import Operator, Panel, PropertyGroup, UIList, AddonPreferences


# Translation function
def tr(text_en, text_zh):
    lang = bpy.context.preferences.view.language
    if lang.startswith('zh'):
        return text_zh
    else:
        return text_en


# Updated object types list to include all 17 types
object_types = [
    ('MESH', 'assign_mesh', tr('Mesh', '网格')),
    ('CURVE', 'assign_curve', tr('Curve', '曲线')),
    ('SURFACE', 'assign_surface', tr('Surface', '表面')),
    ('META', 'assign_meta', tr('Meta', '融球')),
    ('FONT', 'assign_font', tr('Font', '字体')),
    ('VOLUME', 'assign_volume', tr('Volume', '体积')),
    ('GPENCIL', 'assign_gpencil', tr('Grease Pencil', '蜡笔')),
    ('ARMATURE', 'assign_armature', tr('Armature', '骨架')),
    ('LATTICE', 'assign_lattice', tr('Lattice', '晶格')),
    ('EMPTY', 'assign_empty', tr('Empty', '空对象')),
    ('IMAGE', 'assign_image', tr('Image', '图像')),
    ('LIGHT', 'assign_light', tr('Light', '灯光')),
    ('LIGHT_PROBE', 'assign_light_probe', tr('Light Probe', '光照探头')),
    ('CAMERA', 'assign_camera', tr('Camera', '摄像机')),
    ('SPEAKER', 'assign_speaker', tr('Speaker', '扬声器')),
    ('FORCE_FIELD', 'assign_force_field', tr('Force Field', '力场')),
    ('COLLECTION_INSTANCE', 'assign_collection_instance', tr('Collection Instance', '集合实例')),
]


def get_collection_names():
    """Return collection names in the current language."""
    collection_names = {}
    obj_type_property = bpy.types.Object.bl_rna.properties['type']
    obj_type_items = obj_type_property.enum_items

    # Get localized names for object types in enum_items
    for obj_type in [
        'MESH', 'CURVE', 'SURFACE', 'META', 'FONT', 'VOLUME', 'GPENCIL', 'ARMATURE', 'LATTICE',
        'EMPTY', 'LIGHT', 'LIGHT_PROBE', 'CAMERA', 'SPEAKER',
    ]:
        name = obj_type_items[obj_type].name
        localized_name = bpy.app.translations.pgettext_iface(name)
        collection_names[obj_type] = localized_name

    # Handle 'IMAGE' (Empty display type)
    empty_display_property = bpy.types.Object.bl_rna.properties['empty_display_type']
    empty_display_items = empty_display_property.enum_items
    image_name = empty_display_items['IMAGE'].name
    localized_image_name = bpy.app.translations.pgettext_iface(image_name)
    collection_names['IMAGE'] = localized_image_name

    # Handle 'COLLECTION_INSTANCE' (Object instance_type)
    instance_type_property = bpy.types.Object.bl_rna.properties['instance_type']
    instance_type_items = instance_type_property.enum_items
    collection_instance_name = instance_type_items['COLLECTION'].name
    localized_collection_instance_name = bpy.app.translations.pgettext_iface(collection_instance_name)
    collection_names['COLLECTION_INSTANCE'] = localized_collection_instance_name

    # Handle 'FORCE_FIELD' (Force Field)
    force_field_name = tr("Force Field", "力场")
    localized_force_field_name = bpy.app.translations.pgettext_iface(force_field_name)
    collection_names['FORCE_FIELD'] = localized_force_field_name

    return collection_names


def create_type_property(prop_name, display_name):
    def update(self, context):
        # Ensure only one parent collection has this type assigned
        if getattr(self, prop_name):
            for pcoll in context.scene.organize_properties.parent_collections:
                if pcoll != self:
                    if getattr(pcoll, prop_name):
                        setattr(pcoll, prop_name, False)
    return BoolProperty(name=display_name, update=update)


class ParentCollectionPropertyGroup(PropertyGroup):
    name: StringProperty(name=tr("Collection Name", "集合名称"), default=tr("New Collection", "新建集合"))
    unique_id: StringProperty(name=tr("Unique ID", "唯一ID"))

    # Dynamically create properties
    pass  # We'll add properties via __annotations__


# Ensure __annotations__ exists
if not hasattr(ParentCollectionPropertyGroup, '__annotations__'):
    ParentCollectionPropertyGroup.__annotations__ = {}

# Add type properties to the PropertyGroup with update functions
for obj_type, prop_name, display_name in object_types:
    prop = create_type_property(prop_name, display_name)
    ParentCollectionPropertyGroup.__annotations__[prop_name] = prop


class MaterialSelectionPropertyGroup(PropertyGroup):
    material_name: StringProperty(name=tr("Material Name", "材质名称"))
    select: BoolProperty(name=tr("Select", "选择"), default=False)


def has_physics_properties(obj):
    if hasattr(obj, 'rigid_body') and obj.rigid_body is not None:
        return True
    if hasattr(obj, 'collision') and obj.collision is not None:
        return True
    if hasattr(obj, 'modifiers'):
        for modifier in obj.modifiers:
            if modifier.type in {'CLOTH', 'SOFT_BODY', 'FLUID', 'SMOKE'}:
                return True
    if hasattr(obj, 'field') and obj.field is not None and obj.field.type != 'NONE':
        return True
    if hasattr(obj, 'dynamic_paint') and obj.dynamic_paint is not None:
        return True
    return False


class ORGANIZE_OT_OrganizeCollections(Operator):
    """Organize objects into collections based on type, material properties, physics properties, and parent-child relationships, ensuring each type is assigned to only one parent collection."""
    bl_idname = "object.organize_collections"
    bl_label = tr("Organize Collections", "组织集合")
    bl_options = {'REGISTER', 'UNDO'}

    _timer = None
    _start_time = None
    _duration = 3
    _steps = []
    _current_step = 0
    _created_collections = set()
    _existing_collections_before = set()
    _cancelled = False

    def execute(self, context):
        try:
            self.context = context  # Save context for use in helper methods
            self.enable_emission_to_light = context.scene.organize_properties.enable_emission_to_light
            self.organize_selected_only = context.scene.organize_properties.organize_selected_only
            self.big_collection_active = context.scene.organize_properties.big_collection_active
            self.parent_child_active = context.scene.organize_properties.parent_child_active
            self.material_based_grouping_active = context.scene.organize_properties.material_based_grouping_active
            self.physics_based_grouping_active = context.scene.organize_properties.physics_based_grouping_active

            # Clean empty collections before starting a new organization
            self.clean_empty_collections()

            self._existing_collections_before = set(bpy.data.collections.keys())
            self._prepare_steps(context)

            total_steps = len(self._steps)
            # Retrieve dynamic display time from addon preferences
            addon_prefs = context.preferences.addons[__name__].preferences
            self._duration = min(addon_prefs.dynamic_display_time, total_steps * 0.1) if total_steps > 0 else addon_prefs.dynamic_display_time

            self._start_time = time.time()
            self._timer = context.window_manager.event_timer_add(0.1, window=context.window)
            context.window_manager.modal_handler_add(self)

            # Initialize progress bar
            if total_steps > 0:
                context.window_manager.progress_begin(0, total_steps)
            else:
                self.report({'INFO'}, tr("No steps to execute.", "没有步骤需要执行。"))
                return {'FINISHED'}

            return {'RUNNING_MODAL'}
        except Exception as e:
            self.report({'ERROR'}, tr("Error during organization: ", "组织过程中出错：") + str(e))
            return {'CANCELLED'}

    def clean_empty_collections(self):
        """Remove empty collections from the scene."""
        collections_to_delete = []
        for collection in bpy.data.collections:
            if collection.name == self.context.scene.collection.name:
                continue  # Skip the master collection
            if not collection.objects and not collection.children:
                collections_to_delete.append(collection)

        for collection in collections_to_delete:
            try:
                # Unlink from all parent collections
                for parent in bpy.data.collections:
                    if collection.name in parent.children.keys():
                        parent.children.unlink(collection)
                # Remove collection
                bpy.data.collections.remove(collection)
            except ReferenceError as e:
                self.report({'ERROR'}, tr(f"Failed to delete collection '{collection.name}': ", f"无法删除集合 '{collection.name}'：") + str(e))

    def _create_collection(self, name, parent_collection=None):
        if name in bpy.data.collections:
            return bpy.data.collections[name]
        else:
            try:
                new_collection = bpy.data.collections.new(name)
                if parent_collection:
                    parent_collection.children.link(new_collection)
                else:
                    self.context.scene.collection.children.link(new_collection)
                return new_collection
            except Exception as e:
                self.report({'ERROR'}, tr(f"Failed to create collection '{name}': ", f"无法创建集合 '{name}'：") + str(e))
                return None

    def _assign_object_to_collection(self, obj_name, collection_name):
        obj = bpy.data.objects.get(obj_name)
        collection = bpy.data.collections.get(collection_name)
        if obj and collection:
            if collection not in obj.users_collection:
                try:
                    # Unlink from all other collections
                    for col in obj.users_collection[:]:
                        col.objects.unlink(obj)
                    collection.objects.link(obj)
                except Exception as e:
                    self.report({'ERROR'}, tr(f"Failed to assign object '{obj_name}' to collection '{collection_name}': ", f"无法将对象 '{obj_name}' 分配到集合 '{collection_name}'：") + str(e))

    def _prepare_steps(self, context):
        self._steps = []
        self._created_collections = set()
        type_based_objects = {}
        parent_child_objects = {}
        material_based_objects = {}
        physics_based_objects = []

        supported_types = {item[0] for item in object_types}

        objs_to_process = bpy.context.selected_objects if self.organize_selected_only else bpy.data.objects

        # Gather selected materials if material-based grouping is active
        selected_materials = set()
        if self.material_based_grouping_active:
            for mat_sel in context.scene.organize_properties.selected_materials:
                if mat_sel.select:
                    selected_materials.add(mat_sel.material_name)

        # Set to keep track of already assigned objects to prevent conflicts
        assigned_objects = set()

        for obj in objs_to_process:
            if obj is None:
                continue  # Skip if the object is None

            obj_type = obj.type

            # Handle collection instances, images, and force fields
            if obj_type == 'EMPTY':
                if getattr(obj, 'instance_type', None) == 'COLLECTION':
                    obj_type = 'COLLECTION_INSTANCE'
                elif obj.empty_display_type == 'IMAGE':
                    obj_type = 'IMAGE'
            elif hasattr(obj, 'field') and obj.field is not None and obj.field.type != 'NONE':
                obj_type = 'FORCE_FIELD'

            if obj_type == 'MESH' and is_single_emission_material(obj):
                if self.enable_emission_to_light:
                    obj_type = 'LIGHT'

            if obj_type not in supported_types:
                continue

            # Material-Based Grouping
            if self.material_based_grouping_active and hasattr(obj.data, 'materials'):
                materials = obj.data.materials
                if materials:
                    for mat in materials:
                        if mat and mat.name in selected_materials:
                            mat_name = mat.name
                            if mat_name not in material_based_objects:
                                material_based_objects[mat_name] = []
                            material_based_objects[mat_name].append(obj)

            # Physics-Based Grouping
            if self.physics_based_grouping_active and has_physics_properties(obj):
                physics_based_objects.append(obj)

            # Parent-Child Organization
            if self.parent_child_active and obj.parent:
                parent = obj.parent
                if parent not in parent_child_objects:
                    parent_child_objects[parent] = []
                parent_child_objects[parent].append(obj)
            else:
                if obj_type not in type_based_objects:
                    type_based_objects[obj_type] = []
                type_based_objects[obj_type].append(obj)

        # Get the collection names (localized)
        collection_names = get_collection_names()

        # Priority 1: Material-Based Grouping
        if self.material_based_grouping_active:
            for mat_name, objs in material_based_objects.items():
                for obj in objs:
                    if obj.name in assigned_objects:
                        continue  # Skip already assigned objects
                    # Search for existing material collection with organize_material == mat_name
                    material_collection = None
                    for col in context.scene.collection.children:
                        if col.get('organize_material') == mat_name:
                            material_collection = col
                            break

                    if material_collection is None:
                        material_collection_name = f"Material_{mat_name}"
                        material_collection = self._create_collection(material_collection_name)
                        if material_collection:
                            material_collection['organize_material'] = mat_name
                            self._steps.append(('create_collection', material_collection.name))
                            self._created_collections.add(material_collection.name)
                        else:
                            return

                    if material_collection:
                        self._steps.append(('assign_object', material_collection.name, obj.name))
                        assigned_objects.add(obj.name)

        # Priority 2: Physics-Based Grouping
        if self.physics_based_grouping_active:
            for obj in physics_based_objects:
                if obj.name in assigned_objects:
                    continue  # Skip already assigned objects

                # Search for existing physics collection
                physics_collection = None
                for col in context.scene.collection.children:
                    if col.get('organize_physics') == True:
                        physics_collection = col
                        break

                if physics_collection is None:
                    physics_collection_name = collection_names.get('FORCE_FIELD', tr("Physics", "物理"))
                    physics_collection = self._create_collection(physics_collection_name)
                    if physics_collection:
                        physics_collection['organize_physics'] = True
                        self._steps.append(('create_collection', physics_collection.name))
                        self._created_collections.add(physics_collection.name)
                    else:
                        return

                self._steps.append(('assign_object', physics_collection.name, obj.name))
                assigned_objects.add(obj.name)

        # Priority 3: Parent-Child Organization
        if self.parent_child_active:
            for parent, children in parent_child_objects.items():
                # Check if parent is already assigned via material-based or physics-based grouping
                if parent.name in assigned_objects:
                    continue  # Skip if parent is already assigned

                parent_type = parent.type

                # Determine the big collection for the parent
                big_collection_id = None
                if self.big_collection_active:
                    for pcoll in context.scene.organize_properties.parent_collections:
                        for obj_type, prop_name, display_name in object_types:
                            if getattr(pcoll, prop_name) and parent_type == obj_type:
                                big_collection_id = pcoll.unique_id
                                break
                        if big_collection_id:
                            break

                if big_collection_id:
                    # Search for existing big_collection with organize_big_collection_id == big_collection_id
                    big_collection = None
                    for col in context.scene.collection.children:
                        if col.get('organize_big_collection_id') == big_collection_id:
                            big_collection = col
                            break

                    if big_collection is None:
                        # Get the name from the parent collection property
                        pcoll = None
                        for pc in context.scene.organize_properties.parent_collections:
                            if pc.unique_id == big_collection_id:
                                pcoll = pc
                                break
                        if pcoll is None:
                            self.report({'ERROR'}, tr("Failed to find parent collection property.", "未能找到父子级属性。"))
                            return

                        big_collection_name = pcoll.name

                        big_collection = self._create_collection(big_collection_name)
                        if big_collection:
                            big_collection['organize_big_collection_id'] = big_collection_id
                            self._steps.append(('create_collection', big_collection.name))
                            self._created_collections.add(big_collection.name)
                        else:
                            return

                    # Create a group collection for the parent
                    # Search for existing parent_group collection with organize_parent_object == parent.name
                    parent_group = None
                    for col in big_collection.children:
                        if col.get('organize_parent_object') == parent.name:
                            parent_group = col
                            break

                    if parent_group is None:
                        parent_group_name = f"{parent.name}_Group"
                        parent_group = self._create_collection(parent_group_name, parent_collection=big_collection)
                        if parent_group:
                            parent_group['organize_parent_object'] = parent.name
                            self._steps.append(('create_collection', parent_group.name, big_collection.name))
                            self._created_collections.add(parent_group.name)
                        else:
                            return

                    # Assign parent and children to the parent_group
                    if parent.name not in assigned_objects:
                        self._steps.append(('assign_object', parent_group.name, parent.name))
                        assigned_objects.add(parent.name)
                    for child in children:
                        if child.name not in assigned_objects:
                            self._steps.append(('assign_object', parent_group.name, child.name))
                            assigned_objects.add(child.name)
                else:
                    # If big_collection_active is not enabled, assign to scene collection
                    # Search for existing parent_group collection with organize_parent_object == parent.name
                    parent_group = None
                    for col in context.scene.collection.children:
                        if col.get('organize_parent_object') == parent.name:
                            parent_group = col
                            break

                    if parent_group is None:
                        parent_group_name = f"{parent.name}_Group"
                        parent_group = self._create_collection(parent_group_name)
                        if parent_group:
                            parent_group['organize_parent_object'] = parent.name
                            self._steps.append(('create_collection', parent_group.name))
                            self._created_collections.add(parent_group.name)
                        else:
                            return

                    # Assign parent and children to the parent_group
                    if parent.name not in assigned_objects:
                        self._steps.append(('assign_object', parent_group.name, parent.name))
                        assigned_objects.add(parent.name)
                    for child in children:
                        if child.name not in assigned_objects:
                            self._steps.append(('assign_object', parent_group.name, child.name))
                            assigned_objects.add(child.name)

        # Priority 4: Type-Based Organization
        if self.big_collection_active:
            # Collect types assigned to big collections
            assigned_types = {}
            for pcoll in context.scene.organize_properties.parent_collections:
                big_collection_id = pcoll.unique_id
                for obj_type, prop_name, display_name in object_types:
                    if getattr(pcoll, prop_name):
                        if obj_type in assigned_types:
                            # This should not happen due to the update function
                            self.report({'ERROR'}, tr(f"Type '{display_name}' is assigned to multiple parent collections.", f"类型 '{display_name}' 被分配给多个父子级。"))
                            return
                        assigned_types[obj_type] = big_collection_id

            for obj_type, objs in type_based_objects.items():
                if obj_type in assigned_types:
                    big_collection_id = assigned_types[obj_type]

                    # Search for existing big_collection with organize_big_collection_id == big_collection_id
                    big_collection = None
                    for col in context.scene.collection.children:
                        if col.get('organize_big_collection_id') == big_collection_id:
                            big_collection = col
                            break

                    if big_collection is None:
                        # Get the name from the parent collection property
                        pcoll = None
                        for pc in context.scene.organize_properties.parent_collections:
                            if pc.unique_id == big_collection_id:
                                pcoll = pc
                                break
                        if pcoll is None:
                            self.report({'ERROR'}, tr("Failed to find parent collection property.", "未能找到父子级属性。"))
                            return

                        big_collection_name = pcoll.name

                        big_collection = self._create_collection(big_collection_name)
                        if big_collection:
                            big_collection['organize_big_collection_id'] = big_collection_id
                            self._steps.append(('create_collection', big_collection.name))
                            self._created_collections.add(big_collection.name)
                        else:
                            return

                    # Now, search for type_collection under big_collection
                    type_collection = None
                    for col in big_collection.children:
                        if col.get('organize_type') == obj_type:
                            type_collection = col
                            break

                    if type_collection is None:
                        type_collection_name = collection_names[obj_type]
                        type_collection = self._create_collection(type_collection_name, parent_collection=big_collection)
                        if type_collection:
                            type_collection['organize_type'] = obj_type
                            self._steps.append(('create_collection', type_collection.name, big_collection.name))
                            self._created_collections.add(type_collection.name)
                        else:
                            return

                    for obj in objs:
                        if obj.name in assigned_objects:
                            continue  # Skip already assigned objects
                        if type_collection not in obj.users_collection:
                            self._steps.append(('assign_object', type_collection.name, obj.name))
                            assigned_objects.add(obj.name)
                else:
                    # Assign to default type collection under scene collection
                    # Search for existing type_collection with organize_type == obj_type
                    type_collection = None
                    for col in context.scene.collection.children:
                        if col.get('organize_type') == obj_type:
                            type_collection = col
                            break

                    if type_collection is None:
                        type_collection_name = collection_names[obj_type]
                        type_collection = self._create_collection(type_collection_name)
                        if type_collection:
                            type_collection['organize_type'] = obj_type
                            self._steps.append(('create_collection', type_collection.name))
                            self._created_collections.add(type_collection.name)
                        else:
                            return

                    for obj in objs:
                        if obj.name in assigned_objects:
                            continue  # Skip already assigned objects
                        if type_collection not in obj.users_collection:
                            self._steps.append(('assign_object', type_collection.name, obj.name))
                            assigned_objects.add(obj.name)

        else:
            # Default organization without big collections
            for obj_type, objs in type_based_objects.items():
                # Search for existing type_collection with organize_type == obj_type
                type_collection = None
                for col in context.scene.collection.children:
                    if col.get('organize_type') == obj_type:
                        type_collection = col
                        break

                if type_collection is None:
                    type_collection_name = collection_names[obj_type]
                    type_collection = self._create_collection(type_collection_name)
                    if type_collection:
                        type_collection['organize_type'] = obj_type
                        self._steps.append(('create_collection', type_collection.name))
                        self._created_collections.add(type_collection.name)
                    else:
                        return

                for obj in objs:
                    if obj.name in assigned_objects:
                        continue  # Skip already assigned objects
                    if type_collection not in obj.users_collection:
                        self._steps.append(('assign_object', type_collection.name, obj.name))
                        assigned_objects.add(obj.name)

        self._steps.append(('delete_empty_collections',))

    def modal(self, context, event):
        if event.type == 'ESC' and event.value == 'PRESS':
            self._cancelled = True
            self.report({'INFO'}, tr("Cancellation", "取消组织"))
            context.window_manager.progress_end()
            context.window_manager.event_timer_remove(self._timer)
            return {'CANCELLED'}

        if event.type == 'TIMER':
            try:
                elapsed = time.time() - self._start_time
                total_steps = len(self._steps)
                if total_steps == 0:
                    context.window_manager.progress_end()
                    context.window_manager.event_timer_remove(self._timer)
                    return {'FINISHED'}
                step_interval = self._duration / total_steps if total_steps > 0 else self._duration
                if step_interval == 0:
                    step_interval = 0.1  # Avoid division by zero
                step_number = int(elapsed / step_interval)

                while step_number > self._current_step and self._current_step < total_steps:
                    step = self._steps[self._current_step]
                    if step[0] == 'create_collection':
                        if len(step) == 3:
                            collection_name, parent_name = step[1], step[2]
                            parent_collection = bpy.data.collections.get(parent_name)
                            if parent_collection:
                                self._create_collection(collection_name, parent_collection=parent_collection)
                        else:
                            collection_name = step[1]
                            self._create_collection(collection_name)
                    elif step[0] == 'assign_object':
                        collection_name, obj_name = step[1], step[2]
                        self._assign_object_to_collection(obj_name, collection_name)
                    elif step[0] == 'delete_empty_collections':
                        # Collect collections to delete
                        collections_to_delete = []
                        for collection in bpy.data.collections:
                            if collection.name == context.scene.collection.name:
                                continue
                            if not collection.objects and not collection.children:
                                collections_to_delete.append(collection)

                        # Delete collected collections
                        for collection in collections_to_delete:
                            try:
                                # Unlink from all parent collections
                                for parent in bpy.data.collections:
                                    if collection.name in parent.children.keys():
                                        parent.children.unlink(collection)
                                # Remove collection
                                bpy.data.collections.remove(collection)
                            except ReferenceError as e:
                                self.report({'ERROR'}, tr(f"Failed to delete collection '{collection.name}': ", f"无法删除集合 '{collection.name}'：") + str(e))

                    self._current_step += 1
                    context.window_manager.progress_update(self._current_step)

                if elapsed >= self._duration or self._current_step >= total_steps or self._cancelled:
                    context.window_manager.progress_end()
                    context.window_manager.event_timer_remove(self._timer)
                    if self._cancelled:
                        return {'CANCELLED'}
                    return {'FINISHED'}
            except Exception as e:
                self.report({'ERROR'}, tr("Error during organization: ", "组织过程中出错：") + str(e))
                context.window_manager.progress_end()
                context.window_manager.event_timer_remove(self._timer)
                return {'CANCELLED'}

        return {'PASS_THROUGH'}


class ORGANIZE_OT_DeleteAllCollections(Operator):
    bl_idname = "object.delete_all_collections"
    bl_label = tr("Delete All Collections", "删除所有集合")
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        try:
            all_objects = list(bpy.data.objects)
            for obj in all_objects:
                if context.scene.collection not in obj.users_collection:
                    context.scene.collection.objects.link(obj)
            collections_to_delete = [col for col in bpy.data.collections if col != context.scene.collection]
            for col in collections_to_delete:
                bpy.data.collections.remove(col)
            self.report({'INFO'}, tr("All collections deleted", "所有集合已删除"))
            return {'FINISHED'}
        except Exception as e:
            self.report({'ERROR'}, tr("Error deleting collections: ", "删除集合时出错：") + str(e))
            return {'CANCELLED'}


class ORGANIZE_OT_AddParentCollection(Operator):
    bl_idname = "organize.add_parent_collection"
    bl_label = tr("Add custom Collection", "添加自定义集合")

    def execute(self, context):
        organize_props = context.scene.organize_properties
        new_pcoll = organize_props.parent_collections.add()
        new_pcoll.name = tr("Collection", "集合")
        new_pcoll.unique_id = uuid.uuid4().hex
        organize_props.parent_collections_index = len(organize_props.parent_collections) - 1
        return {'FINISHED'}


class ORGANIZE_OT_RemoveParentCollection(Operator):
    bl_idname = "organize.remove_parent_collection"
    bl_label = tr("Remove custom Collection", "移除自定义集合")

    def execute(self, context):
        organize_props = context.scene.organize_properties
        index = organize_props.parent_collections_index
        if index < 0 or index >= len(organize_props.parent_collections):
            self.report({'ERROR'}, tr("Invalid custom collection index.", "无效的自定义集合索引。"))
            return {'CANCELLED'}
        # Reset type assignments in the collection being removed
        pcoll = organize_props.parent_collections[index]
        for obj_type, prop_name, display_name in object_types:
            setattr(pcoll, prop_name, False)
        organize_props.parent_collections.remove(index)
        if index > 0:
            organize_props.parent_collections_index -= 1
        else:
            organize_props.parent_collections_index = 0
        return {'FINISHED'}


class ORGANIZE_UL_ParentCollections(UIList):
    def draw_item(self, context, layout, data, item, icon, active_data, active_propname, index):
        pcoll = item
        if self.layout_type in {'DEFAULT', 'COMPACT'}:
            layout.prop(pcoll, "name", text="", emboss=False, icon='OUTLINER_COLLECTION')
        elif self.layout_type in {'GRID'}:
            layout.alignment = 'CENTER'
            layout.prop(pcoll, "name", text="", emboss=False)


def contains_chinese(text):
    for ch in text:
        if '\u4e00' <= ch <= '\u9fff':
            return True
    return False


class ORGANIZE_OT_RefreshMaterialList(Operator):
    bl_idname = "organize.refresh_material_list"
    bl_label = tr("Refresh Material List", "刷新材质列表")
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        organize_props = context.scene.organize_properties
        # Preserve currently selected materials
        currently_selected = {mat_sel.material_name for mat_sel in organize_props.selected_materials if mat_sel.select}

        organize_props.selected_materials.clear()
        # Get all materials
        all_materials = bpy.data.materials[:]
        # Sort materials: Chinese names first
        all_materials.sort(key=lambda mat: (not contains_chinese(mat.name), mat.name))

        for mat in all_materials:
            mat_sel = organize_props.selected_materials.add()
            mat_sel.material_name = mat.name
            mat_sel.select = mat.name in currently_selected  # Restore selection if previously selected

        self.report({'INFO'}, tr("Material list refreshed.", "材质列表已刷新。"))
        return {'FINISHED'}

class ORGANIZE_UL_SelectedMaterials(UIList):
    def filter_items(self, context, data, propname):
        items = getattr(data, propname)
        helper_funcs = bpy.types.UI_UL_list
        flt_flags = []
        flt_neworder = []
        if self.filter_name:
            flt_flags = helper_funcs.filter_items_by_name(self.filter_name, self.bitflag_filter_item, items, "material_name", reverse=False)
        else:
            flt_flags = [self.bitflag_filter_item] * len(items)
        return flt_flags, flt_neworder

    def draw_item(self, context, layout, data, item, icon, active_data, active_propname, index):
        mat_sel = item
        mat = bpy.data.materials.get(mat_sel.material_name)
        
        row = layout.row(align=True)
        row.prop(mat_sel, "select", text="", toggle=True)
        
        if mat and mat.preview and hasattr(mat.preview, 'icon_id') and mat.preview.icon_id:
            # 使用 icon_value 传递整数类型的图标ID
            row.label(text="", icon_value=mat.preview.icon_id)
        else:
            # 使用 icon 传递预定义的图标名称
            row.label(text="", icon='MATERIAL')
        
        row.label(text=mat_sel.material_name)

class ORGANIZE_PT_Panel(Panel):
    """Creates a Panel in the 3D View Sidebar"""
    bl_label = tr("Auto Organize", "自动组织集合")
    bl_idname = "ORGANIZE_PT_panel"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = tr('Auto Organize', '自动组织集合')

    def draw(self, context):
        layout = self.layout
        scene = context.scene
        organize_props = scene.organize_properties

        # Actions box
        action_box = layout.box()
        action_box.label(text=tr("Actions", "操作"), icon='TOOL_SETTINGS')
        action_col = action_box.column(align=True)
        action_col.operator("object.organize_collections", text=tr("Organize Collections", "组织集合"), icon='OUTLINER_OB_GROUP_INSTANCE')
        action_col.operator("object.delete_all_collections", text=tr("Delete All Collections", "删除所有集合"), icon='TRASH')

        # Options box
        box = layout.box()
        box.label(text=tr("Options", "选项"), icon='PREFERENCES')
        col = box.column(align=True)
        col.prop(organize_props, "enable_emission_to_light", text=tr("Emitting to Light", "将自发光材质标记为灯光"), icon='LIGHT_DATA')
        col.prop(organize_props, "organize_selected_only", text=tr("Selected Only", "仅选择"), icon='RESTRICT_SELECT_OFF')
        col.prop(organize_props, "parent_child_active", text=tr("Parent Collection", "父子级"), toggle=True, icon='ORIENTATION_PARENT')
        col.prop(organize_props, "physics_based_grouping_active", text=tr("Physics Collection", "物理集合"), toggle=True, icon='PHYSICS')

        # Big Collection toggle
        col.prop(organize_props, "big_collection_active", text=tr("Custom Collection", "自定义集合"), toggle=True, icon='OUTLINER_COLLECTION')

        # Material-Based Grouping toggle
        col.prop(organize_props, "material_based_grouping_active", text=tr("Material Collection", "材质集合"), toggle=True, icon='MATERIAL')

        if organize_props.big_collection_active or organize_props.parent_child_active or organize_props.material_based_grouping_active:
            if organize_props.big_collection_active:
                # Parent Collections
                col.label(text=tr("Custom Collection:", "自定义集合："), icon='OUTLINER_COLLECTION')
                row = col.row()
                row.template_list("ORGANIZE_UL_ParentCollections", "", organize_props, "parent_collections", organize_props, "parent_collections_index")

                col_operators = row.column(align=True)
                col_operators.operator("organize.add_parent_collection", icon='ADD', text="")
                col_operators.operator("organize.remove_parent_collection", icon='REMOVE', text="")

                if organize_props.parent_collections:
                    pcoll = organize_props.parent_collections[organize_props.parent_collections_index]

                    # Type assignments box
                    type_box = col.box()
                    type_box.label(text=tr("Collection type", "集合类型"), icon='OBJECT_DATA')
                    col_types = type_box.column(align=True)
                    num_columns = 2
                    rows = (len(object_types) + num_columns - 1) // num_columns
                    for row_idx in range(rows):
                        row = col_types.row(align=True)
                        for col_idx in range(num_columns):
                            idx = row_idx * num_columns + col_idx
                            if idx < len(object_types):
                                obj_type, prop_name, display_name = object_types[idx]
                                row.prop(pcoll, prop_name, text=display_name, toggle=True)

            if organize_props.material_based_grouping_active:
                # Material-Based Grouping Section
                col.label(text=tr("Material-Based Grouping:", "材质集合："), icon='MATERIAL')
                row = col.row(align=True)
                row.scale_x = 0.3
                row.operator("organize.refresh_material_list", icon='FILE_REFRESH', text=tr("Refresh", "刷新"))
                row = col.row()
                row.template_list("ORGANIZE_UL_SelectedMaterials", "", organize_props, "selected_materials", organize_props, "material_selection_index")


class ORGANIZE_OT_RegisterProperties(PropertyGroup):
    enable_emission_to_light: BoolProperty(
        name=tr("Assign Self-Emitting Objects to Light Collection", "将自发光对象分配到灯光集合"),
        description=tr("Decide whether to put objects with self-emitting materials into the Light collection", "将具有自发光材质的对象放入灯光集合"),
        default=True
    )
    organize_selected_only: BoolProperty(
        name=tr("Organize Selected Objects Only", "仅组织选中的对象"),
        description=tr("When enabled, only organize selected objects", "启用时，仅组织选中的对象"),
        default=False
    )
    big_collection_active: BoolProperty(
        name=tr("Custom Collection Active", "自定义集合激活"),
        description=tr("Toggle to activate or deactivate custom collection functions", "切换以激活或停用自定义集合功能"),
        default=False
    )
    parent_child_active: BoolProperty(
        name=tr("Parent-Child Organization Active", "父子组织激活"),
        description=tr("Toggle to activate or deactivate parent-child based organization", "切换以激活或停用基于父子关系的组织"),
        default=False
    )
    material_based_grouping_active: BoolProperty(
        name=tr("Material-Based Grouping Active", "基于材质的分组激活"),
        description=tr("Toggle to activate or deactivate material-based grouping of objects", "切换以激活或停用基于材质的对象分组"),
        default=False
    )
    physics_based_grouping_active: BoolProperty(
        name=tr("Physics-Based Grouping Active", "基于物理的分组激活"),
        description=tr("Toggle to activate or deactivate physics-based grouping of objects", "切换以激活或停用基于物理的对象分组"),
        default=False
    )
    parent_collections: CollectionProperty(type=ParentCollectionPropertyGroup)
    parent_collections_index: IntProperty()

    selected_materials: CollectionProperty(type=MaterialSelectionPropertyGroup)
    material_selection_index: IntProperty()


class ORGANIZE_AddonPreferences(AddonPreferences):
    bl_idname = __name__

    dynamic_display_time: FloatProperty(
        name=tr("Dynamic Display Time", "动态显示时间"),
        description=tr("Time in seconds for dynamic display", "动态显示的时间（秒）"),
        default=2.0,
        min=0.1,
        max=10.0,
        step=1,
        precision=1
    )

    def draw(self, context):
        layout = self.layout
        layout.prop(self, "dynamic_display_time", text=tr("Dynamic Display Time (s)", "动态显示时间（秒）"))


def is_single_emission_material(obj):
    if obj.type != 'MESH':
        return False
    if not obj.data:
        return False
    if not hasattr(obj.data, 'materials'):
        return False
    if not obj.data.materials:
        return False
    for mat in obj.data.materials:
        if mat is None or not mat.use_nodes:
            return False
        emission_nodes = [node for node in mat.node_tree.nodes if node.type == 'EMISSION']
        if len(emission_nodes) != 1:
            return False
        links = mat.node_tree.links
        emission_node = emission_nodes[0]
        connected = False
        for link in links:
            if hasattr(link.to_node, 'type') and link.from_node == emission_node and link.to_node.type == 'OUTPUT_MATERIAL':
                connected = True
                break
        if not connected:
            return False
    return True


def register():
    bpy.utils.register_class(ParentCollectionPropertyGroup)
    bpy.utils.register_class(MaterialSelectionPropertyGroup)
    bpy.utils.register_class(ORGANIZE_OT_OrganizeCollections)
    bpy.utils.register_class(ORGANIZE_OT_DeleteAllCollections)
    bpy.utils.register_class(ORGANIZE_OT_AddParentCollection)
    bpy.utils.register_class(ORGANIZE_OT_RemoveParentCollection)
    bpy.utils.register_class(ORGANIZE_UL_ParentCollections)
    bpy.utils.register_class(ORGANIZE_OT_RefreshMaterialList)
    bpy.utils.register_class(ORGANIZE_UL_SelectedMaterials)
    bpy.utils.register_class(ORGANIZE_PT_Panel)
    bpy.utils.register_class(ORGANIZE_OT_RegisterProperties)
    bpy.utils.register_class(ORGANIZE_AddonPreferences)
    bpy.types.Scene.organize_properties = PointerProperty(type=ORGANIZE_OT_RegisterProperties)


def unregister():
    bpy.utils.unregister_class(ParentCollectionPropertyGroup)
    bpy.utils.unregister_class(MaterialSelectionPropertyGroup)
    bpy.utils.unregister_class(ORGANIZE_OT_OrganizeCollections)
    bpy.utils.unregister_class(ORGANIZE_OT_DeleteAllCollections)
    bpy.utils.unregister_class(ORGANIZE_OT_AddParentCollection)
    bpy.utils.unregister_class(ORGANIZE_OT_RemoveParentCollection)
    bpy.utils.unregister_class(ORGANIZE_UL_ParentCollections)
    bpy.utils.unregister_class(ORGANIZE_OT_RefreshMaterialList)
    bpy.utils.unregister_class(ORGANIZE_UL_SelectedMaterials)
    bpy.utils.unregister_class(ORGANIZE_PT_Panel)
    bpy.utils.unregister_class(ORGANIZE_OT_RegisterProperties)
    bpy.utils.unregister_class(ORGANIZE_AddonPreferences)
    del bpy.types.Scene.organize_properties


if __name__ == "__main__":
    register()
